'''
Пользовательский скрипт для первой ректификации спирта-сырца
--------------------------------------------------------------------------------
История изменений в конце файла
OldBean, 25.07.2021
'''
#-------------------------------------------------------------------------------
from lite import *
#-------------------------------------------------------------------------------
''' Для работы в локальной сети, небходимо разремовать следующую строку и 
задать IP-адрес малинки, на которой будет активирован TCP-сервер, через который
клиенткие приложения могут удаленно мониторить процесс и управлять им.
В противном случае TCP-сервер будет работать в локальном режиме на малинке и
клиентское приложение может быть запущено тоже только на малинке.
'''
#server_addr = ('xxx.xxx.xxx.xxx', 5238)
#-------------------------------------------------------------------------------
'''Для отправки сообщений в телеграм-чат, необходимо разремовать следующие две
строки и задать реальный токен телеграм-бота и id чата. См. инструкцию по 
созданию телеграм-ботов на сайте telegram.
'''
#token = '1111111111:AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
#chat_id = 1234567890

url = 'https://api.telegram.org/bot' + token + '/'
u.bot_flag = True if token and chat_id else False # Флаг отсылки сообщений боту
if not u.bot_flag: # Добавим переменную в список скрытых переменных
  uhide.append('bot_flag')
#-------------------------------------------------------------------------------
s0.v = 0 # Фракционник - в нулевое состоянии
s0.delay = 5 # Замедлим скорость вращения сервы
#-------------------------------------------------------------------------------
''' Определяем пользовательские переменные. Эти пользовательские переменные
будут находиться в контейнере u (объект класса Cont). Этим и определяется
необходимость использования префикса u при создании переменных.
'''
u.Qg = 200.0 # Количество отбираемых голов, мл
u.Qpg = 1000.0 # Количество отбираемых подголовников, мл

u.T1cr = 65.0 # Температура в колонне при котором прекращается режим разгона
u.T2cr = 78.6 # Т-ра в дефлегматоре окончания отбора исправимых хвостов
u.T3cr = 50.0 # Критическая т-ра в ТСА -> аварийное отключение
u.T4cr = 40.0 # Критическая т-ра воды охлаждения -> аварийное отключение

u.ww = 600.0 # Рабочая мощность нагрева куба, Вт
u.qg = 50.0 # Скорость отбора голов, мл/час
u.qw = 400.0 # Рабочая скорость отбора (подголовники, тело, хвосты), мл/час

u.dT = 0.0625; uhide.append('dT') # Квант датчика DS18B20, скрытая переменная
u.T1max = 0.0 # Максимальная температура в царге во время отбора подголовников
u.Tset = u.Tsb = 0.0 # Текущее и базовое значения уставки старт-стопа
u.Pb = 0.0 # Атм.давление в момент фиксации базового значения уставки
u.ss_flag = False # Флаг режима старт/стопа
u.tsa_flag = False # Флаг контроля температуры ТСА
u.w_flag = True # Флаг контроля температуры воды охлаждения
u.ss_cntr = 0 # Количество случившихся старт/стопов в данном процессе
u.timing = []; uhide.append('timing') # Моменты стопов и стартов, скрытая пер-я
#-------------------------------------------------------------------------------
''' Далее необходимо определить режимы работы установки - создать объекты
класса Mode. Для каждого режима необходимо создать две функции.

Первая функция (здесь ее название finit()) вызывается один раз при переключении
системы в данный режим работы. В ней происходит инициализация нужных системных
и пользовательских переменных.

Вторая функция (здесь fcond()) вызывается периодически в главном цикле
приложения (см. модуль main.py). В ней производится проверка условий
переключения в другой режим работы системы или завершения работы установки.

Порядок создания режимов важен, т.к. его номер является горячей клавишей для
перехода в этот режим. Режим мониторинга, который предусмотрен в системе всегда,
имеет номер 0. Следующие режимы создаются пользователем. Их нумерация
начинается с 1.
'''
#-------------------------------------------------------------------------------
# Определение режима разгона (режим 1)
def finit(): # Инициализация режима
  w0.on() # Включаем нагрев ТЭНа на максимум
  q0.off() # Отбор выключен
  Q0.v = 0.0 # Сброс виртуального датчика расхода
  u.ss_flag = False # Старт-стоп в этом режиме не используется
  s0.v = 0 # Фракционник - в нулевое состоянии
def fcond(): # Проверка условий
  if T1.v > u.T1cr: # Температура в колонне превысила пороговую
    u.mode = 2 # перейдем в режим "холостого хода"
modes.append(Mode('Разгон', finit, fcond))
#-------------------------------------------------------------------------------
# Определение режима холостого хода после разгона (режим 2)
def finit(): # Инициализация режима
  w0.v = u.ww # Нагрев ТЭНа на рабочем уровне
  q0.off() # Отбор выключен
  Q0.v = 0.0 # Сброс виртуального датчика расхода
  u.ss_flag = False # Старт-стоп в этом режиме не используется
  s0.v = 0 # Фракционник - в нулевое состоянии
def fcond(): # Проверка условий
  #!!!!!!!!!!!!!!!!!! Вставить проверку стационарности T3 и T3 !!!!!!!!!!!!
  # Данная реализация - просто по времени. Обычно это около 30 мин
  if time.time() - tim.mst > 1800: # 30 мин прошло
    u.mode = 3 # Переходим в режим отбора голов
  pass
modes.append(Mode('Холостой ход', finit, fcond))
#-------------------------------------------------------------------------------
# Определение режима отбора голов (режим 3)
def finit(): # Инициализация режима
  w0.v = u.ww # Нагрев ТЭНа на рабочем уровне
  q0.v = u.qg # Скорость отбора голов
  Q0.v = 0.0 # Сброс виртуального датчика расхода
  u.ss_flag = False # Старт-стоп в этом режиме не используется
s0.v = 36 # Поток направлен в емкость для приема голов (°)
  time.sleep(1)
def fcond(): # Проверка условий
  if Q0.v > u.Qg: # Отобрано нужное количество голов
    u.mode = 4 # Переход на режим отбора подголовников
modes.append(Mode('Головы', finit, fcond))
#-------------------------------------------------------------------------------
# Определение режима отбора подголовников (режим 4)
def finit(): # Инициализация режима
  w0.v = u.ww # Нагрев ТЭНа на рабочем уровне
  q0.v = u.qw # Рабочая скорость отбора
  Q0.v = 0.0 # Сброс виртуального датчика расхода
  u.T1max = 0.0 # Рабочая пер-я для оценки максимальной т-ры в колонне
  u.ss_flag = False # Старт-стоп в этом режиме не используется
  s0.v = 72 # Поток направлен в емкость приема подголовников (°)
  time.sleep(1)
def fcond(): # Проверка условий
  ''' Максимальная температура в колонне на этапе отбора подголовников нужна
      для автоматической установки уставки старт/стопа на следующем этапе
      (отбор тела)
  '''
  if T1.v > u.T1max: # Оценка максимальной т-ры в колонне
    u.T1max = T1.v
  if Q0.v > u.Qpg: # Отобрано нужное количество подголовников
    u.mode = 5 # Переход на режим отбора тела
modes.append(Mode('Подголовники', finit, fcond))
#-------------------------------------------------------------------------------
# Определение режима отбора тела (режим 5)
def finit(): # Инициализация режима
  w0.v = u.ww # Нагрев ТЭНа на рабочем уровне
  q0.v = u.qw # Рабочая скорость отбора
  Q0.v = 0.0 # Сброс виртуального датчика расхода
  ''' Уставка задается на 5 квантов выше максимальной температуры в колонне,
      полученной на предыдущем этапе (отбор подголовников). Обычно этого
      хватает...
  '''
  u.Tset = max(u.T1max, T1.v) + 5*u.dT
  u.Tsb = u.Tset # Пер-е для коррекции уставки на атм. давление
  u.Pb = P0.v
  u.ss_flag = True # Включим режим старт-стопа
  u.ss_cntr = 0 # Обнуляем счетчик старт/стопов
  u.timing.clear()
  s0.v = 108 # Поток направлен в емкость для приема тела (°)
  time.sleep(1)
def fcond(): # Проверка условий
  u.Tset = u.Tsb + 0.033*(P0.v - u.Pb) # Коррекция уставки
  if u.ss_flag:
    ch_flag = False
    if q0.v == 0.0 and T1.v <= u.Tset - u.dT: # Условие для старта
      q0.v = u.qw
      ch_flag = True
    if q0.v > 0.0 and T1.v >= u.Tset + u.dT: # Условие для стопа
      q0.v = 0.0
      ch_flag = True
    if ch_flag: # Была смена состояния клапана отбора
      u.ss_cntr += 1 # Модифицируем счетчик старт/стопов
      u.timing.append(time.time())
      if len(u.timing) > 2: # Подсчитаем эффективную скорость отбора
        if q0.v > 0: # Был старт
          koef = u.timing[-2] - u.timing[-3]
        else:
          koef = u.timing[-1] - u.timing[-2]
        koef /= u.timing[-1] - u.timing[-3]
#        u.q0_eff = int(u.qw*koef)
        u.q0_eff = f'{100*koef:.0f}%'
    if u.ss_cntr > 40: # Количество старт-стопов превысило предел
      u.mode = 6 # Переход в режим отбора оборотных хвостов
modes.append(Mode('Тело', finit, fcond))
#-------------------------------------------------------------------------------
# Определение режима отбора оборотных хвостов (режим 6)
def finit(): # Инициализация режима
  w0.v = u.ww # Нагрев ТЭНа на рабочем уровне
  q0.v = u.qw # Рабочая скорость отбора
  Q0.v = 0.0 # Сброс виртуального датчика расхода
  u.ss_flag = False # Старт-стоп в этом режиме не используется
  s0.v = 144 # Поток направлен в емкость для приема оборотных хвостов (°)
  time.sleep(1)
def fcond(): # Проверка условий
  if T2.v > u.T2cr: # Температура пара в дефлегматоре начала расти
    u.mode = 0
modes.append(Mode('Хвосты', finit, fcond))
#-------------------------------------------------------------------------------
def emergency(): # Проверка аварийных ситуаций
  ret = False
  if u.w_flag and T4.v > u.T4cr: # Проверка температуры воды охлаждения
    ret = True
  if u.tsa_flag and T3.v > u.T3cr: # Проверка температуры ТСА
    ret = True
  return ret
#-------------------------------------------------------------------------------
''' В этом блоке кода определяются специфичные для процесса пользовательские
горячие клавиши. Нельзя использовать клавиши h, q, k, m, u, v, которые уже
используются библиотекой lite. Новые клавишы добавляются в общий словарь
горячих клавиш библиотеки lite - hot_keys.
'''
for nm, m in enumerate(modes): # Добавим горячие клавиши (это цифры, номера
  hot_keys[str(nm)] = m.name   # определенных выше режимов работы установки

''' Добавим специфичные для данного процесса горячие клавиши
'''
hot_keys['g'] = 'Задать уставку по текущей температуре'
hot_keys['+'] = 'Увеличить уставку'
hot_keys['-'] = 'Уменьшить уставку'
hot_keys['s'] = 'Включить/выключить режим старт/стоп'
hot_keys['t'] = 'Включить/выключить контроль температуры ТСА'
hot_keys['w'] = 'Включить/выключить контроль температуры воды'
hot_keys['b'] = 'Посылать или нет важную информацию на чат'
#-------------------------------------------------------------------------------
''' В этом блоке определяется функционал, связанный с каждой пользовательской
горячей клавишей. Функционал оформлен в виде функции user_hk(), которая
вызывается периодически в главном цикле приложения (см. модуль main.py).
Параметром функции является символ нажатой клавиши. Функция возвращает строку
сообщения, которое будет отправлено пользователю (на чат или клиентскому
приложению).
'''
def user_hk(ch): # Отработка пользовательских горячих клавиш
  msg = '' # Сообщение, которое будет отправлено клиенту
  if ch == 'g': # Уставке старт-стопа присвоить текущее значение т-ры
    u.Tset = T1.v
    u.Tsb = u.Tset
    u.Pb = P0.v
    msg = f'u.Tset = {u.Tset:.3f}'
  elif ch == '+': # Увеличить на квант текущее значение уставки
    u.Tset += u.dT
    u.Tsb = u.Tset
    u.Pb = P0.v
    msg = f'u.Tset = {u.Tset:.3f}'
  elif ch == '-': # Уменьшить на квант текущее значение уставки
    u.Tset -= u.dT
    u.Tsb = u.Tset
    u.Pb = P0.v
    msg = f'u.Tset = {u.Tset:.3f}'
  elif ch == 's': # Включить/выключить старт-стоп
    u.ss_flag = not u.ss_flag
    msg = f'u.ss_flag = {u.ss_flag}'
  elif ch == 't': # Включить/выключить контроль температуры ТСА
    u.tsa_flag = not u.tsa_flag
    msg = f'u.tsa_flag = {u.tsa_flag}'
  elif ch == 'w': # Включить/выключить контроль температуры воды
    u.w_flag = not u.w_flag
    msg = f'u.w_flag = {u.w_flag}'
  elif ch == 'b': # Включить/выключить отсылку сообщений 
    if token and chat_id: # Если чат доступен
      if u.bot_flag:
        msg = 'Отсылка сообщений будет выключена'
        crit_info(msg)
      u.bot_flag = not u.bot_flag
      if u.bot_flag:
        msg = 'Отсылка сообщений включена'
        crit_info(msg)
      msg = f'u.bot_flag = {u.bot_flag}'
  return msg
#-------------------------------------------------------------------------------
#Добавляем список переменных, которые нужно писать в лог
u.log_head += 'Tset '
u.log_vars += '{u.Tset:.3f}'
#-------------------------------------------------------------------------------
def crit_info(message): # Callback функция, вызываемая при важных событиях
  if u.bot_flag: # Включен режим отсылки сообщений на чат
    tele_send_msg(url, chat_id, message)
#-------------------------------------------------------------------------------
''' Список изменений
25.07.2021 1. Косметические правки, добавление комментариев
'''
#-------------------------------------------------------------------------------

