'''
Пользовательский скрипт для первой ректификации спирта-сырца
--------------------------------------------------------------------------------
История изменений в конце файла
OldBean, 25.07.2021
'''
#-------------------------------------------------------------------------------
from lite import *
#-------------------------------------------------------------------------------
''' Для работы в локальной сети, небходимо разремовать следующую строку и 
задать IP-адрес малинки, на которой будет активирован TCP-сервер, через который
клиенткие приложения могут удаленно мониторить процесс и управлять им.
В противном случае TCP-сервер будет работать в локальном режиме на малинке и
клиентское приложение может быть запущено тоже только на малинке.
'''
#server_addr = ('xxx.xxx.xxx.xxx', 5238)
#-------------------------------------------------------------------------------
'''Для отправки сообщений в телеграм-чат, необходимо разремовать следующие две
строки и задать реальный токен телеграм-бота и id чата. См. инструкцию по 
созданию телеграм-ботов на сайте telegram.
'''
#token = '1111111111:AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
#chat_id = 1234567890

url = 'https://api.telegram.org/bot' + token + '/'
u.bot_flag = True if token and chat_id else False # Флаг отсылки сообщений боту
if not u.bot_flag: # Добавим переменную в список скрытых переменных
  uhide.append('bot_flag')
#-------------------------------------------------------------------------------
s0.v = 0 # Фракционник - в нулевое состоянии
s0.delay = 5 # Замедлим скорость вращения сервы
#-------------------------------------------------------------------------------
''' Определяем пользовательские переменные. Эти пользовательские переменные
будут находиться в контейнере u (объект класса Cont). Этим и определяется
необходимость использования префикса u при создании переменных.
'''
u.Qg = 200.0 # Количество отбираемых голов, мл
u.Q2 = 17000.0 # Количество отбираемого СР2, мл
u.Qpp = 500.0 # Кол-во прошедшей воды (пара) для пропарки, мл

u.T1cr = 65.0 # Температура в колонне при котором прекращается режим разгона
u.T3cr = 50.0 # Критическая т-ра в ТСА - аварийное отключение
u.T4cr = 40.0 # Критическая т-ра воды охлаждения - аварийное отключение

u.ww = 600.0 # Рабочая мощность нагрева куба, Вт
u.qg = 50.0 # Скорость отбора голов, мл/час
u.q2 = 200.0 # Рабочая скорость отбора (CP2), мл/час

u.tsa_flag = False # Флаг контроля температуры ТСА
u.w_flag = True # Флаг контроля температуры воды охлаждения
#-------------------------------------------------------------------------------
''' Далее необходимо определить режимы работы установки - создать объекты
класса Mode. Для каждого режима необходимо создать две функции.

Первая функция (здесь ее название finit()) вызывается один раз при переключении
системы в данный режим работы. В ней происходит инициализация нужных системных
и пользовательских переменных.

Вторая функция (здесь fcond()) вызывается периодически в главном цикле
приложения (см. модуль main.py). В ней производится проверка условий
переключения в другой режим работы системы или завершения работы установки.

Порядок создания режимов важен, т.к. его номер является горячей клавишей для
перехода в этот режим. Режим мониторинга, который предусмотрен в системе всегда,
имеет номер 0. Следующие режимы создаются пользователем. Их нумерация
начинается с 1.
'''
#-------------------------------------------------------------------------------
# Определение режима разгона (режим 1)
def finit(): # Инициализация режима
  w0.on() # Включаем нагрев ТЭНа на максимум
  q0.off() # Отбор выключен
  Q0.v = 0.0 # Сброс виртуального датчика расхода
  s0.v = 0 # Фракционник - в нулевое состоянии
def fcond(): # Проверка условий
  if T1.v > u.T1cr: # Температура в колонне превысила пороговую
    u.mode = 2 # перейдем в режим "холостого хода"
modes.append(Mode('Разгон', finit, fcond))
#-------------------------------------------------------------------------------
# Определение режима холостого хода после разгона (режим 2)
def finit(): # Инициализация режима
  w0.v = u.ww # Нагрев ТЭНа на рабочем уровне
  q0.off() # Отбор выключен
  Q0.v = 0.0 # Сброс виртуального датчика расхода
  s0.v = 0 # Фракционник - в нулевое состоянии
def fcond(): # Проверка условий
  if time.time() - tim.mst > 1800: # Выход по времени (через 30 мин)
    u.mode = 3
  pass
modes.append(Mode('Холостой ход', finit, fcond))
#-------------------------------------------------------------------------------
# Определение режима отбора голов (режим 3)
def finit(): # Инициализация режима
  w0.v = u.ww # Нагрев ТЭНа на рабочем уровне
  q0.v = u.qg # Скорость отбора голов
  Q0.v = 0.0 # Сброс виртуального датчика расхода
  s0.v = 36 # Поток направлен в емкость для приема голов (°)
  time.sleep(1)
def fcond(): # Проверка условий
  if Q0.v > u.Qg: # Отобрано нужное количество голов
    u.mode = 4 # Переход в режим отбора тела (СР-2)
modes.append(Mode('Головы', finit, fcond))
#-------------------------------------------------------------------------------
# Определение режима отбора тела СР2 (режим 4)
def finit(): # Инициализация режима
  w0.v = u.ww # Нагрев ТЭНа на рабочем уровне
  q0.v = u.q2 # Рабочая скорость отбора
  Q0.v = 0.0 # Сброс виртуального датчика расхода
  s0.v = 108 # Отбор СР-2 по тому же каналу, что и тело для СР-1
  time.sleep(1)
def fcond(): # Проверка условий
  if Q0.v > u.Q2: # Отобрано нужное СР-2
    u.mode = 0 # Все выключаем и уходим в режим мониторинга
modes.append(Mode('СР2', finit, fcond))
#-------------------------------------------------------------------------------
# Определение режима пропарки колонны (режим 5)
def finit(): # Инициализация режима
  w0.v = u.ww
  q0.on()
  Q0.v = 0.0
  u.tsa_flag = False
  s0.v = 144 # Слив в отдельную емкость
  time.sleep(1)
def fcond(): # Проверка условий
  if Q0.v > u.Qpp:
    u.mode = 0 # Все выключаем - режим мониторинга
modes.append(Mode('Пропарка колонны', finit, fcond))
#-------------------------------------------------------------------------------
# Определение режима открыть клапан (режим 6)
def finit(): # Инициализация режима
  # Мощность нагрева не изменяется (остается от предыдущего режима)
  q0.on() # Клапан полностью открывается
  Q0.v = 0.0
  u.tsa_flag = False
  s0.v = 180 # Слив в отдельную емкость
  time.sleep(1)
def fcond(): # Проверка условий
  pass # Условий автоматического выхода из режима нет
modes.append(Mode('Открыть клапан', finit, fcond))
#-------------------------------------------------------------------------------
def emergency(): # Проверка аварийных ситуаций
  ret = False
  if u.w_flag and T4.v > u.T4cr: # Проверка температуры воды охлаждения
    ret = True
  if u.tsa_flag and T3.v > u.T3cr: # Проверка температуры ТСА
    ret = True
  return ret
#-------------------------------------------------------------------------------
''' В этом блоке кода определяются специфичные для процесса пользовательские
горячие клавиши. Нельзя использовать клавиши h, q, k, m, u, v, которые уже
используются библиотекой lite. Новые клавишы добавляются в общий словарь
горячих клавиш библиотеки lite - hot_keys.
'''
for nm, m in enumerate(modes): # Добавим горячие клавиши для установки режимов,
  hot_keys[str(nm)] = m.name   # определенных выше

''' Добавим специфичные для данного процесса горячие клавиши
'''
hot_keys['t'] = 'Включить/выключить контроль температуры ТСА'
hot_keys['w'] = 'Включить/выключить контроль температуры воды'
hot_keys['b'] = 'Посылать или нет важную информацию на чат'
#-------------------------------------------------------------------------------
''' В этом блоке определяется функционал, связанный с каждой пользовательской
горячей клавишей. Функционал оформлен в виде функции user_hk(), которая
вызывается периодически в главном цикле приложения (см. модуль main.py).
Параметром функции является символ нажатой клавиши. Функция возвращает строку
сообщения, которое будет отправлено пользователю (на чат или клиентскому
приложению).
'''
def user_hk(ch): # Отработка пользовательских горячих клавиш
  msg = '' # Сообщение, которое будет отправлено клиенту
  if ch == 't':
    u.tsa_flag = not u.tsa_flag
    msg = f'u.tsa_flag = {u.tsa_flag}'
  elif ch == 'w':
    u.w_flag = not u.w_flag
    msg = f'u.w_flag = {u.w_flag}'
  elif ch == 'b':
    if token and chat_id: # Если чат доступен
      if u.bot_flag:
        msg = 'Отсылка сообщений будет выключена'
        crit_info(msg)
      u.bot_flag = not u.bot_flag
      if u.bot_flag:
        msg = 'Отсылка сообщений включена'
        crit_info(msg)
      msg = f'u.bot_flag = {u.bot_flag}'
  return msg
#-------------------------------------------------------------------------------
def crit_info(message): # Callback функция, вызываемая при важных событиях
  if u.bot_flag: # Включен режим отсылки сообщений на чат
    tele_send_msg(url, chat_id, message)
#-------------------------------------------------------------------------------
''' Список изменений
25.07.2021 1. Косметические правки, добавление комментариев
'''
#-------------------------------------------------------------------------------


